/* ----------------------------------------------------------------------------
 *  $RCSfile: igmpv4.c,v $
 * 
 * $Revision: 1.6 $
 *   $Author: larse $
 *     $Date: 2000/06/07 16:43:17 $
 *    $State: Exp $
 * ----------------------------------------------------------------------------
 * Copyright 1999-2000, Lars Eggert <larse@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License for more details.
 * ----------------------------------------------------------------------------
 *
 * $Log: igmpv4.c,v $
 * Revision 1.6  2000/06/07 16:43:17  larse
 * Moved software over to sourceforge.net under the Artistic License. No
 * functional changes to latest release.
 *
 * Revision 1.5  1999/08/13 23:50:09  larse
 * Nicer header separation; fixed quirk with printing newlines.
 *
 * Revision 1.4  1999/05/20 21:20:08  larse
 * Now compiles independent of inet includes. (First step towards
 * multi-platform support.)
 *
 * Revision 1.3  1999/05/18 00:47:24  larse
 * Added ARP/RARP and DVMRP. Factored out common code to basics.c. Added
 * IGMP traceroute (untested).
 *
 * Revision 1.2  1999/05/12 13:04:59  larse
 * Added support for SAP and SAP. Removed duplicate includes from other
 * sources. Changed output of IP router alert option in last version, but
 * forgot to document.
 *
 * Revision 1.1  1999/05/11 17:45:05  larse
 * Initial support for IGMPv4, mostly missing is support for DVMRP.
 *
 * ----------------------------------------------------------------------------
 */


/* Define a marker for ident(1). */
static const char rcsid[] = 
  "$Id: igmpv4.c,v 1.6 2000/06/07 16:43:17 larse Exp $";


#include <stdio.h>
#include <netdb.h>

#include "ipdump.h"
#include "indent.h"
#include "scan.h"


struct igmp {
  u_char          igmp_type;      /* version & type of IGMP message  */
  u_char          igmp_code;      /* subtype for routing msgs        */
  u_short         igmp_cksum;     /* IP-style checksum               */
  u_long	  igmp_group;     /* group address being reported    */
};				  /*  (zero for queries)             */

#define IGMP_MEMBERSHIP_QUERY           0x11    /* membership query         */
#define IGMP_V1_MEMBERSHIP_REPORT       0x12    /* Ver. 1 membership report */
#define IGMP_V2_MEMBERSHIP_REPORT       0x16    /* Ver. 2 membership report */
#define IGMP_V2_LEAVE_GROUP             0x17    /* Leave-group message      */

#define IGMP_DVMRP                      0x13    /* DVMRP routing message    */
#define IGMP_PIM                        0x14    /* PIM routing message      */

#define IGMP_MTRACE_RESP                0x1e  /* traceroute resp.(to sender)*/
#define IGMP_MTRACE                     0x1f  /* mcast traceroute messages  */

#define IGMP_MAX_HOST_REPORT_DELAY   10    /* max delay for response to     */
                                           /*  query (in seconds) according */
                                           /*  to RFC1112                   */


/* local header id */
static const char hdr[] = "IGMPv4";


/* print an igmp header */
void print_igmpv4(void)
  {
  const struct igmp *ih = (const struct igmp *)next(sizeof(struct igmp));

  if(ih == 0) return;
  iprintf(hdr, "type: %hu ", ih->igmp_type);
  switch(ih->igmp_type)
    {
    case IGMP_MEMBERSHIP_QUERY: printf("[membership query]\n"); break;
    case IGMP_V1_MEMBERSHIP_REPORT: printf("[v1 membership report]\n"); break;
    case IGMP_V2_MEMBERSHIP_REPORT: printf("[v2 membership report]\n"); break;
    case IGMP_V2_LEAVE_GROUP: printf("[leave group]\n"); break;
    case IGMP_DVMRP: printf("[dvmrp]\n"); break;
    case IGMP_PIM: printf("[pim]\n"); break;
    case IGMP_MTRACE_RESP: printf("[traceroute response]\n"); break;
    case IGMP_MTRACE: printf("[traceroute]\n"); break;
    case IGMP_MAX_HOST_REPORT_DELAY: 
      printf("[maximum host report delay]\n"); break;
    default: 
      unsupported++;
      printf("[*unsupported*]\n");
    }

  switch(ih->igmp_type)
    {
    case IGMP_MEMBERSHIP_QUERY:
    case IGMP_V1_MEMBERSHIP_REPORT:
    case IGMP_V2_MEMBERSHIP_REPORT:
    case IGMP_V2_LEAVE_GROUP:
      iprintf(hdr, "max response time: %hu [%.1f seconds]\n", 
	      ntohs(ih->igmp_code), ntohs(ih->igmp_code) / 10.0);
      iprintf(hdr, "checksum: %hu\n", ntohs(ih->igmp_cksum));
      print_address(hdr, "group address", (u_long *)&ih->igmp_group);
      printf("\n");
      break;

    case IGMP_DVMRP:
      unget(sizeof(struct igmp));
      print_seperator();
      print_dvmrp();
      break;

    case IGMP_MTRACE:
      {
      const u_long *addr;
      const u_short *cksum;
      const u_char *oct = next(sizeof(u_char));
      if(oct == 0) return;
      iprintf(hdr, "hops: %hu\n", *oct);
      cksum = (u_short *)next(sizeof(u_short));
      if(cksum == 0) return;
      iprintf(hdr, "checksum: %hu\n", ntohs(*cksum));
      addr = (u_long *)next(sizeof(u_long));
      if(addr == 0) return;
      print_address(hdr, "group address", addr);
      printf("\n");
      addr = (u_long *)next(sizeof(u_long));
      if(addr == 0) return;
      print_address(hdr, "source address", addr);
      printf("\n");
      addr = (u_long *)next(sizeof(u_long));
      if(addr == 0) return;
      print_address(hdr, "destination address", addr);
      printf("\n");
      addr = (u_long *)next(sizeof(u_long));
      if(addr == 0) return;
      print_address(hdr, "response address", addr);
      printf("\n");
      addr = (u_long *)next(sizeof(u_long));
      if(addr == 0) return;
      iprintf(hdr, "response ttl: %hu\n", *oct & 0xff000000);
      iprintf(hdr, "query id: %lu\n", ntohl(*oct & 0x00ffffff));
      break;
      }

    default:
      unsupported++;
      iprintf(hdr, "[*unsupported*]\n");
    }
  }
